//
//  DrawPanel.java
//  Halo
//
//  Created by lars on 24/03/2008.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import static java.lang.Math.*;
import java.util.*;

public class DrawPanel extends JPanel /*implements ActionListener*/ {
	public double x, y, z;
	public boolean hit;
	public double[] scatVec = new double[4];
	public double[] cVec = new double[4];
	public double[] inVec = new double[4];
	Random random = new Random();
	public double dx;
	
	
	public DrawPanel() {
	}
	
/************************************************************************/
	public void RotationY() {
		double temp = x;
		x = x * Halo.cosCoeff[8] + z * Halo.sinCoeff[8];
		z = -temp * Halo.sinCoeff[8] + z * Halo.cosCoeff[8];
	}
	
/************************************************************************/
	public class RotVector {
		public double sinRot;
		public double cosRot;
		public double[] axis = new double[4];
		
		public RotVector() {
			sinRot = 0;
			cosRot = 1;
			for (int i=0; i<4; i++) axis[i] = 0;
		}
		
	}

/************************************************************************/
	public class CStruct {
		public double scat;
		public double[] scatVec = new double[4];
		public double[] cVec = new double[4];
		public double[] inVec = new double[4];
		
	}

/************************************************************************/
	public class OutStruct {
		public double prob;
		public double[] scatVec = new double[4];		
	}

/************************************************************************/
	public RotVector FindRotation(double[] k1,double[] k2){
		//This procedure finds the rotation vector required 
		//to rotate unit vector k1 into unit vector k2.
		RotVector rotation = new RotVector();
		//First find the normalised cross product k1 X k2. That defines the rotation axis
		rotation.sinRot = 0;
		rotation.axis[1] = k1[2]*k2[3]-k1[3]*k2[2];
		rotation.sinRot = rotation.sinRot + rotation.axis[1]*rotation.axis[1];
		rotation.axis[2] = k1[3]*k2[1]-k1[1]*k2[3];
		rotation.sinRot = rotation.sinRot + rotation.axis[2]*rotation.axis[2];	
		rotation.axis[3] = k1[1]*k2[2]-k1[2]*k2[1];
		rotation.sinRot = rotation.sinRot + rotation.axis[3]*rotation.axis[3];
		rotation.sinRot = sqrt(rotation.sinRot);
		//sinRot will be the sine of the rotation angle:	sinRot = |k1 X k2|
	
		rotation.cosRot = 0;
		for (int i=1; i<=3; i++) {
			rotation.cosRot =  rotation.cosRot + k1[i]*k2[i] ;
			// The scalar product k1·k2 is the cosine of the rotation angle
			rotation.axis[i] = rotation.axis[i]/rotation.sinRot; 
			//Make the axis vector a unit vector
		}
		return rotation;
	}
	
/************************************************************************/
	public double[] RotateWith(RotVector omega, double[] inVector) {
		double norm = 0.0;
		double kw = 0.0;
		double[] e1 = new double[4];
		double[] e2 = new double[4];
		//This procedure rotates the inVector using the rotation vector omega
		
		//Find a unit vector perpendicular to omega and the vector
		e2[1] = omega.axis[2]*inVector[3]-omega.axis[3]*inVector[2];
		norm = norm + e2[1]*e2[1];
		e2[2] = omega.axis[3]*inVector[1]-omega.axis[1]*inVector[3];
		norm = norm + e2[2]*e2[2];
		e2[3] = omega.axis[1]*inVector[2]-omega.axis[2]*inVector[1];
		norm = norm + e2[3]* e2[3];
		norm = sqrt(norm);
	
		for (int i=1; i<=3; i++) e2[i] = e2[i]/norm;
	
		//Find the remaining orthogonal base vector
		e1[1] = e2[2]*omega.axis[3]-e2[3]*omega.axis[2];
		e1[2] = e2[3]*omega.axis[1]-e2[1]*omega.axis[3];
		e1[3] = e2[1]*omega.axis[2]-e2[2]*omega.axis[1];

		//Here is the scalar product k·w
		kw = inVector[1]*omega.axis[1]+inVector[2]*omega.axis[2]+inVector[3]*omega.axis[3];
		
		norm = sqrt(1 - kw*kw);
	
		for (int i=1; i<=3; i++) inVector[i] = kw*omega.axis[i] + e1[i]*norm*omega.cosRot + e2[i]*norm*omega.sinRot;
		return inVector;
	}
	
/************************************************************************/
	public CStruct OrientVectorTriad(double[] incidentVector, double[] scatVector, double[] crystalAxis) {
		//This procedure orients the event vectors in a standard way:
		//The incident vector points in the z direction and the
		//scattered vector lies in the xz plane
		//Note that this standard orientation differs from that defined
		//in the paper. Saves some computing time.
		//For Parry simulations the crystal axis is a unit vector
		//perpendicular to a prism face.

		//IMPORTANT!!
		//It is assumed that the raytracing preceding the entering of
		//this procedure has been done with tha incident ray having random 
		//direction and with the crystal symmetry axis 
		//perpendicular to the  basal faces pointing in the z direction and
		//with one prism face perpendicular to the x axis. 

		double norm, scatAngle;
		RotVector omega = new RotVector();
		double[] tempVector1 = new double[4];
		double[] tempVector2 = new double[4];
		double[] zAxis = new double[4];
	
		//Set the z direction
		zAxis[1] = 0.0;
		zAxis[2] = 0.0;
		zAxis[3] = 1.0;

		//Find rotation to bring the incident ray to the positive z direction
		omega = FindRotation(incidentVector, zAxis);
	
		//Rotate the triad with the found rotation}
		crystalAxis = RotateWith(omega, crystalAxis);
		scatVector = RotateWith(omega, scatVector);
		incidentVector = RotateWith(omega, incidentVector);
		
		//The third vector in the triad is then fixed
		incidentVector[1] = 0.0;
		incidentVector[2] = 0.0;
		incidentVector[3] = 1.0;

		//Project the scatter vector into the xy plane
		norm = sqrt(scatVector[1]*scatVector[1]+scatVector[2]*scatVector[2]);
		tempVector1[1] = scatVector[1]/norm;
		tempVector1[2] = scatVector[2]/norm;
		tempVector1[3] = 0;
	
		//This is a unit x-vector, the target vector
		tempVector2[1] = 1.0;
		tempVector2[2] = 0.0;
		tempVector2[3] = 0.0;	
	
		//Find the z axis rotation to bring the scatter vector into the xz plane
		omega = FindRotation(tempVector1, tempVector2);

		//and rotate the event vectors with the found rotation
		crystalAxis = RotateWith(omega, crystalAxis);
		scatVector = RotateWith(omega, scatVector);
	
		//The triad is now in a standard orientation and fully defined
		//i.e the incident vector in the z direction, the scatter vector in the xz plane
		//and we are ready to enter the next procedure.
	
		//Compute the scattering angle
		scatAngle = 0.0;
		for (int i=1; i<=3; i++) scatAngle = scatAngle + incidentVector[i]*scatVector[i];

		scatAngle = atan(sqrt(1-scatAngle*scatAngle)/scatAngle);
		if (scatAngle < 0)  scatAngle = scatAngle+PI;
		
		CStruct cc = new CStruct();
		cc.scat = scatAngle;
		for (int i=1; i<4; i++) {
			cc.scatVec[i] = scatVector[i];
			cc.cVec[i] = crystalAxis[i];
			cc.inVec[i] = incidentVector[i];
		}	
		return cc;
	}
		
/************************************************************************/
	public OutStruct CigarEventProbability(double scatAngle, double[] crystalAxis, double[] observer, double[] scatVector, double targetTip) {
		//This is the main procedure. Using the incident direction (0,0,1), 
		//and the scattering angle (scatAngle)a point is generated on 
		//the surface of the Minnaert cigar. 
		//Then we find the rotation around the y axis necessary to make the incident 
		//direction point at this point and rotate the crystal axis 
		//and the scattered ray using this rotation. We then rotate 
		//the crystal axis and the scattered ray randomly around the
		//axis "observer" defined by the vector from the sourceto the observer. 
		//We then find the probability that the crystal tip now has the orientation
		//defined by targetTip. 
	
		//This is the procedure that recycles every event a number of times
		//proportional to the cigar weight.
	
		double h;
		double theta;
		double alpha;
		double s, c, temp;
		double phi;
		double tip;
		double probability;
		RotVector omega = new RotVector();
	
		//Generate a latitudinal point on the cigar
		h = random.nextDouble();
		theta = scatAngle*h;
		dx = sin(theta)/sin(scatAngle);
		//Find the rotation angle to bring the incident ray to this point
		alpha = -PI/2 - atan(observer[3]/observer[1]) - theta;
		c = cos(alpha);
		s = sin(alpha);
	
		//Rotate the scattered ray and the crystal axis in the xz plane using this angle
		temp = scatVector[1];
		scatVector[1] =  temp*c + scatVector[3]*s;
		scatVector[3] = -temp*s + scatVector[3]*c;
	
		temp = crystalAxis[1];
		crystalAxis[1] =  temp*c + crystalAxis[3]*s;
		crystalAxis[3] = -temp*s + crystalAxis[3]*c;

		//Generate a rotation angle for an axial rotation on the cigar
		phi = 2*PI* random.nextDouble();
	
		//Set up a rotation vector. The rotation axis points to the observer
		omega.cosRot = cos(phi);
		omega.sinRot = sin(phi);
		omega.axis[1] = observer[1];
		omega.axis[2] = observer[2];
		omega.axis[3] = observer[3];
	
		//Rotate the scattered ray and the crystal axis using the rotation vector
		//round the axis of the Minnaert cigar
		scatVector = RotateWith(omega, scatVector);
		crystalAxis = RotateWith(omega, crystalAxis);
	
		//Find the tip angle from the vertical
		tip = abs(crystalAxis[3]);  //this is the cosine of the tip angle
		tip = atan(sqrt(1-tip*tip)/tip);
	
		//Check the tip probability distribution and generate the event probability.
		//Weight the event probability with the factor scatAngle/pi.
		//Target tip angles below 40 degrees are here considered to mean plate crystals, 
		//angles above 50 degrees mean column crystals and in the interval
		//40 to 50 degrees mean randomly oriented crystals.
	
		//Parry simulations may require a somewhat more elaborate code
		targetTip = targetTip*PI/180;
		//if (targetTip < 40*PI/180) probability = NormalProbabilityDistribution(0, targetTip, tip);	//Plates
		//else if (targetTip > 50*PI/180) probability = NormalProbabilityDistribution(PI/2, PI/2-targetTip, tip); //Columns
		//else probability = 1.0; //Random crystals
		if (Halo.plateMode == 0 || Halo.plateMode == 2) probability = NormalProbabilityDistribution(0, targetTip, tip);	//Plates
		else probability = NormalProbabilityDistribution(0, targetTip, PI/2-tip); //Columns
		OutStruct result = new OutStruct();
		for (int i=1; i<4; i++) result.scatVec[i] = scatVector[i];
		result.prob = probability;
		return result;	
	}

/************************************************************************/
	public double NormalProbabilityDistribution(double mean, double sigma, double x) {
		//This is just a gaussian with mean = mean and standard deviation = sigma 
		return exp(-(x-mean)/sigma*(x-mean)/sigma/2.0);
	}

/************************************************************************/
	public void paintComponent(Graphics g) {
		double[] observer = new double[4];
		int eventVector = 0;
		
		if(!Halo.firstTime && !Halo.slowUpdate)  {//Shortcut for quick update
			Graphics2D g2d = (Graphics2D)g;
			g2d.drawImage(Halo.image, 0, 0, this);
		} else if (!Halo.firstTime && Halo.slowUpdate) {
			int recycle = 0;
			double targetTip = Halo.tip;//Tip of the crystal
			//Direction to the light source
			observer[1] = cos(Halo.elevation*PI/180);
			observer[2] = 0;
			observer[3] = sin(Halo.elevation*PI/180);
				
			g.setColor(Color.white);
			g.fillRect(0, 0, 560, 590);	//Erase panel
			
			g.setColor(Color.black);
			Halo.eventNumb = 0;
			g.clipRect(10,10,540,540);
			//These procedures are essentially empty. Relics from the PL halo program
			Halo.SetCrystalVectors();
			if (Halo.fisheye) g.fillOval(10,10,540,540);//Paint sky
			else {
				g.fillRect(10,10,540,540);
				g.setColor(Color.red);
				g.drawLine(10, (int)(280 + 270*observer[3]), 550, (int)(280 + 270*observer[3]));
			}
			
			 {//Fisheye
			
				while (Halo.eventNumb < 5000000L) {  //Start the simulation
					eventVector = 1;
					hit = false;
					Halo.event = 0;
					Halo.SetIndex();                 //Set a refraction index
					g.setColor(Halo.cPix);
					Halo.reflNumb = 0;
					Halo.eventNumb = Halo.eventNumb + 1;
					
					if (Halo.Area() / Halo.maxArea > random()) {/////Ray hits crystal but still outside
						hit = true;
						Halo.SelectFace();
						eventVector = eventVector << 3; 
						eventVector = eventVector + (Halo.face + 1)%8;
						if (random() < Halo.Fresnel()) Halo.Reflexion();//Still outside
						else {////We enter crystal
							Halo.Refraction();	
							// Compute polarisations		
							Halo.NewPol(Halo.orthoVec, Halo.rPar, Halo.rOrtho, Halo.tPar, Halo.tOrtho, Halo.polPar, Halo.polOrtho);
							Halo.EntryPoint();		//Get where we enter and note we are inside
							do {					///Travel inside
								Halo.Raytrace();
								eventVector = eventVector << 3; 
								eventVector = eventVector + (Halo.face + 1)%8;
								if (random() < Halo.Fresnel()) Halo.Reflexion();
								else { //We exit the crystal
									Halo.Refraction();
									Halo.inside = false;
								}//
								Halo.NewPol(Halo.orthoVec, Halo.rPar, Halo.rOrtho, Halo.tPar, Halo.tOrtho, Halo.polPar, Halo.polOrtho);
							} while (Halo.inside && Halo.reflNumb < 15); ///Continue as long as we have reflexions and not too many
							if (Halo.reflNumb >= 10) hit = false; 
						}////
					}/////
					
					if (!Halo.inside && hit) {//We are finally outside and have a hit
				
						//Incident ray
						double[] k1 = new double[4];
						//Get it from the main unit
						for (int i=1; i<=3; i++) k1[i] = Halo.k1[i];
						//Normalize – just incase
						double norm = sqrt(k1[1]*k1[1] + k1[2]*k1[2] + k1[3]*k1[3]); 
						for (int i=1; i<=3; i++) k1[i] = k1[i]/norm;
						
						//Processed ray
						double[] rayVector = new double[4];
						//Get it from the main unit
						for (int i=1; i<=3; i++) rayVector[i] = Halo.rayVector[i];
						//Normalize
						norm = sqrt(rayVector[1]*rayVector[1] + rayVector[2]*rayVector[2] + rayVector[3]*rayVector[3]); 
						for (int i=1; i<=3; i++) rayVector[i] = rayVector[i]/norm;
					
						//Scattering angle
						double omega =  k1[1]*rayVector[1]+k1[2]*rayVector[2]+k1[3]*rayVector[3];
						omega = atan(sqrt(1.00000001-omega*omega)/omega);
						if (omega < 0) omega = omega + PI;
						//Determine the reprocessing number, the weight
						if (omega < 0.3*PI/180) recycle = 0;  //Skip rays very close to the source
						else if(omega < PI/180) recycle = Halo.rot; //Rays close to the surface limit of theta/sin(theta)
						else if (omega < 175*PI/180) recycle = (int)(Halo.rot*omega/sin(omega)); //Normal rays
						else recycle = 0; //Skip rays shadowed by the observer's head.
						//Default axis direction
						double[] crystalAxis = new double[4];
						crystalAxis[1]  = 0;
						crystalAxis[2] = 0;
						crystalAxis[3] = 1;
						if (Halo.plateMode == 2) {
							crystalAxis[1]  = 1;
							crystalAxis[3] = 0;
						}
						//Containers for saving an event
						double[] saveRay = new double[4];
						double[] saveIncident = new double[4];
						double[] saveCrystalAxis = new double[4];
						double scatAngle = 0;
				
						if (recycle !=0) {			//We want to repocess
							//Orient vectors in the standard way
							//Incident ray in the z direction. Exit ray in the xz plane
							CStruct cc = OrientVectorTriad(k1, rayVector, crystalAxis); //Orient vetors in the standard way
							for (int i=1; i<4; i++) {
								crystalAxis[i] = cc.cVec[i];
								rayVector[i] = cc.scatVec[i];
								k1[i] = cc.inVec[i];
							}//for
							scatAngle = cc.scat;
							//Save the event for reprocessing
							for(int i=1; i<4; i++) {
								saveRay[i] = rayVector[i];
								saveIncident[i] = k1[i];
								saveCrystalAxis[i] = crystalAxis[i];
							}//for
					
							int locationNumber = 0;
							for (int kk=1; kk<=recycle; kk++) {// Reprocess the event with weight 80*scatAngle/sin(scatAngle);
								for(int i=1; i<4; i++) { //Fetch the event
									k1[i] = saveIncident[i];
									rayVector[i] = saveRay[i];
									crystalAxis[i] = saveCrystalAxis[i];							
								}//for
								//Put in a Minnaert cigar and get the probability given the actual tip versus tipTarget
								OutStruct out = CigarEventProbability(scatAngle, crystalAxis, observer, rayVector, targetTip);
								for (int i=1; i<4; i++) rayVector[i] = out.scatVec[i];
								double prob = out.prob;
								// Plot the event with the proper probability
								if (random.nextDouble() < prob) {
									if (Halo.fisheye) {
										if (rayVector[3] < 0) {//*/
											double gg = sqrt(rayVector[1]*rayVector[1] + rayVector[2]*rayVector[2]);
											int b1 = (int)(atan(gg/rayVector[3])*2/PI*rayVector[1]/gg*270);
											int b2 = (int)(atan(gg/rayVector[3])*2/PI*rayVector[2]/gg*270);
											g.fillOval(b2+280, b1+280, 1, 1);
											Halo.Event e = new Halo.Event();//Create a halo event
											e.pixelVector[0] = eventVector;//Get the ray history
											e.pixelVector[1] = b2 + 280;	//Get the place in the display
											e.pixelVector[2] = b1 + 280;
											e.pixelVector[3] = (int)(dx*10000);
											e.pixelVector[4] = (int)(rayVector[1]*10000);
											e.pixelVector[5] = (int)(rayVector[2]*10000);
											e.pixelVector[6] = (int)(rayVector[3]*10000);
											Halo.eventList.add(e);//Save it
										
										}//if
									} else {
										double x = -rayVector[1];
										double y = -rayVector[2];
										double z = -rayVector[3];
										if (x > 0) {
											double temp = x;
											x = -x*observer[1] - z*observer[3]; //Center sun
											z = temp*observer[3] - z*observer[1];
											z = 270 * z / x;
											y = 270 * y / x;
											int b1 =(int)(y + 0.5 + 280);
											int b2 = (int)(280 - z + 0.5);
											g.fillOval(b1, b2, 1, 1);
											Halo.Event e = new Halo.Event();//Create a halo event
											e.pixelVector[0] = eventVector;//Get the ray history
											e.pixelVector[1] = b1;	//Get the place in the display
											e.pixelVector[2] = b2;
											e.pixelVector[3] = (int)(dx*10000);
											e.pixelVector[4] = (int)(rayVector[1]*10000);
											e.pixelVector[5] = (int)(rayVector[2]*10000);
											e.pixelVector[6] = (int)(rayVector[3]*10000);
											Halo.eventList.add(e);//Save it
										}
									}//else
								}///if
							}///for
						}//if
					}	
				}
				g.setColor(Color.black); //A cross at the position of the sun
				g.drawLine(280, 280 +(270-3*Halo.elevation-4),280, 280 +(270-3*Halo.elevation+4));
				g.drawLine(276, 280+(270-3*Halo.elevation),284, 280+(270-3*Halo.elevation));
				
				Halo.slowUpdate = false;//Enable quick update
				Halo.computationDone = true;
				Halo.saveButton.setEnabled(true);
				Halo.goButton.setEnabled(false);
			}
		}
	}
}
