//
//  HaloViewer.java
//  HaloViewer
//
//  Created by lars on 04/08/2009.
//  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
//
//	For information on setting Java configuration information, including 
//	setting Java properties, refer to the documentation at
//		http://developer.apple.com/techpubs/java/java.html
//

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import static java.lang.Math.*;
import javax.swing.*;
import java.io.*;
import javax.swing.event.*;
import java.util.LinkedList;
import javax.imageio.*;
import java.awt.image.*;

public class HaloViewer extends JFrame {

	private Font font = new Font("serif", Font.ITALIC+Font.BOLD, 36);
	protected AboutBox aboutBox = new AboutBox();
	protected PrefPane prefs;

	static final JMenuBar mb = new JMenuBar();	
	protected JMenu fileMenu = new JMenu("File"); 
	protected JMenuItem quitItem = new JMenuItem("Quit");
	protected JMenuItem aboutItem = new JMenuItem("About HaloViewer...");
	protected JMenuItem importItem = new JMenuItem("Import Halo Data");
	protected JMenuItem saveItem = new JMenuItem("Save Frame");
	protected DrawPanel panel = new DrawPanel();
	protected JSlider azimuthSlider = new JSlider(JSlider.HORIZONTAL, -180, 180, 0);
	protected JSlider elevationSlider = new JSlider(JSlider.VERTICAL, -90, 90, 0);
	public double distance = 5.0;
	public double phi = 0, theta = 0;
	public java.util.List<Data>  dataList = new LinkedList<Data>();;
	static java.util.List<PlotData>  plotList = new LinkedList<PlotData>();
	double zoom = 200;
		
	ChangeListener cl = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == azimuthSlider) {
				phi = azimuthSlider.getValue()*PI/180;
				Compute();
			}
			if (e.getSource() == elevationSlider) {
				theta = elevationSlider.getValue()*PI/180;
				Compute();
			}
		}
	};
								
	MouseWheelListener mwl = new MouseWheelListener() {
		public void mouseWheelMoved(MouseWheelEvent e) {
			int w = e.getWheelRotation();
			zoom = zoom + w*5;
			if (zoom <50) zoom = 50;
			if (zoom > 500) zoom = 500;
			Compute();
		}
	};
	
	ActionListener al = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == quitItem) System.exit(0);
			else if (e.getSource() == importItem) {
				ReadData();
				Compute();
			} else if (e.getSource() == saveItem) SaveFrame(panel, "3DPlot");
			else if (e.getSource() == aboutItem) aboutBox.setVisible(true);
			
		}
	};
		
	public HaloViewer() {
		
		super("");
		setTitle("Halo Viewer");
		this.getContentPane().setLayout(new BorderLayout());
		azimuthSlider.setPaintTicks(true);
		azimuthSlider.setPaintLabels(true);
		azimuthSlider.setMajorTickSpacing(30);
		azimuthSlider.setMinorTickSpacing(10);
		elevationSlider.setPaintTicks(true);
		elevationSlider.setPaintLabels(true);
		elevationSlider.setMajorTickSpacing(30);
		elevationSlider.setMinorTickSpacing(10);
		add(panel, BorderLayout.CENTER);
		add(azimuthSlider, BorderLayout.SOUTH);
		add(elevationSlider, BorderLayout.EAST);
		azimuthSlider.addChangeListener(cl);
		elevationSlider.addChangeListener(cl);
		panel.addMouseWheelListener(mwl);
		addMenus();
		setSize(600, 600);
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
	
/************************************************************/
	public void SaveFrame(Container cp, String name) {
		if (cp == null) return;
		FileDialog fd = new FileDialog(this);
		fd.setMode(FileDialog.SAVE);
		fd.setFile(name);
		fd.setVisible(true);
		String s1 = fd.getFile();
		String s2 = fd.getDirectory();
		name = s2 + s1 +".jpg";
		if (s2 != null) {
			BufferedImage image = new BufferedImage(cp.getWidth(), cp.getHeight(),
			BufferedImage.TYPE_3BYTE_BGR);
			cp.paint(image.getGraphics());
			image = image.getSubimage(0, 0, cp.getWidth(), cp.getHeight());
			try { ImageIO.write(image, "jpg", new File(name)); }
			catch(Throwable thr) { thr.printStackTrace(); }
		}
	}
		
/************************************************************************/
	public void ReadData() {
		FileDialog fd =  new FileDialog(this);
		fd.setMode(FileDialog.LOAD);
		fd.setVisible(true);
		String s1 = fd.getFile();
		String s2 = fd.getDirectory();
		String name = s2 + s1;
		if (s2 != null) {
			dataList.clear();
			try {
				Scanner sc = new Scanner(new File(name));
				while (sc.hasNext()) {
					Data d = new Data();
					String line = sc.nextLine();
					Scanner sc1 = new Scanner(line);
					sc1.useLocale(Locale.US);
					d.pixelVector[0] = sc1.nextDouble();
					d.pixelVector[1] = sc1.nextDouble();
					d.pixelVector[2] = sc1.nextDouble();
					dataList.add(d);
				}
				Compute();
			}
			catch (Exception e) {
				JOptionPane.showMessageDialog(this, "File ERROR", "IOError", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
/************************************************************************/
	static class Data {
		public double[] pixelVector = new double[3];
	}

/************************************************************************/
	static class PlotData {
		public int x; 
		public int y;
	}

/************************************************************************/
	public void Compute() {
		double X = 0, Y = 0, Z = 0, sf = sin(phi), cf = cos(phi), st = sin(theta), ct = cos(theta);
		plotList.clear();
		for(Data d : dataList) {
			X = - distance + (d.pixelVector[0]*cf + d.pixelVector[1]*st)*ct + d.pixelVector[2]*st;
			X = distance/X;
			Y = (d.pixelVector[0]*sf - d.pixelVector[1]*cf)*X;
			Z = ((d.pixelVector[0]*cf + d.pixelVector[1]*sf)*st - d.pixelVector[2]*ct)*X;
			PlotData p = new PlotData();
			p.x = (int)(Y*zoom+300);
			p.y = (int)(300 - Z*zoom);
			plotList.add(p);
		}
		panel.repaint();
	}

/************************************************************************/
	public void addMenus() {
		mb.add(fileMenu);
		fileMenu.add(importItem);
		fileMenu.add(saveItem);
		fileMenu.addSeparator();
		fileMenu.add(aboutItem);
		fileMenu.addSeparator();
		fileMenu.add(quitItem);
		importItem.addActionListener(al);
		saveItem.addActionListener(al);
		quitItem.addActionListener(al);
		aboutItem.addActionListener(al);

		setJMenuBar(mb);
		//setJMenuBar (mb);
	}

/************************************************************************/
	 public static void main(String args[]) {
		new HaloViewer();
	 }

}